#metar-v31.py - (3.1) by Mark Harris. Inspired by JJSilva's NeoSectional Script located; https://github.com/JJSilva/NeoSectional/blob/master/metar.py
#     Added routine by by Nick Cirincione to decode flight category if flight category is not provided by the FAA.
#     Fixed bug that wouldn't allow the last airport to be 'NULL' without causing all LED's to show white.
#     Fixed bug where 'NULL' color would be interpreted as 'nowx'.
#     Added timer routine to allow user to turn off LED's at night.
#     Added internet check with retry incase of power outage and map reboots before router.

#This version retains the added features included in Version 2, hi-wind blinking and lightning when thunderstorms are reported.
#However, this version adds representations for snow, rain, freezing rain, dust sand ash, and fog when reported in the metar.
#The LED's will show the appropriate color for the reported flight category (vfr, mvfr, ifr, lifr) then blink a specific color for the weather
#For instance, an airport reporting IFR with snow would display Red then blink white for a short period to denote snow. Blue for rain,
#purple for freezing rain, brown for dust sand ash, and silver for fog. This makes for a colorful map when weather is in the area.
#A home airport feature has been added as well. When enabled, the map can be dimmed in relation to the home airport as well as
#have the home alternate between weather color and a user defined marker color(s).
#Most of these features can be disabled to downgrade the map display in the user-defined variables below.

#For detailed instructions on building an Aviation Map, visit http://www.livesectional.com
#Hardware features are further explained on this site as well. However, this software allows for a power-on/update weather switch,
#and Power-off/Reboot switch. The use of a display is handled by metar-display.py and not this script.

#Flight Category Definitions. (https://www.aviationweather.gov/taf/help?page=plot)
#+--------------------------------------+---------------+-------------------------------+-------+----------------------------+
#|Category				|Color		|Ceiling	 		|	|Visibility                  |
#|--------------------------------------+---------------+-------------------------------+-------+----------------------------+
#|VFR 	Visual Flight Rules		|Green		|greater than 3,000 feet AGL	|and	|greater than 5 miles        |
#|MVFR 	Marginal Visual Flight Rules	|Blue		|1,000 to 3,000 feet AGL	|and/or	|3 to 5 miles                |
#|IFR 	Instrument Flight Rules		|Red		|500 to below 1,000 feet AGL	|and/or	|1 mile to less than 3 miles |
#|LIFR 	Low Instrument Flight Rules 	|Magenta	|	below 500 feet AGL	|and-or	|less than 1 mile            |
#+--------------------------------------+---------------+-------------------------------+-------+----------------------------+
#AGL = Above Ground Level

#Import needed libraries
import urllib2
import xml.etree.ElementTree as ET
import time
from datetime import datetime
from datetime import timedelta
from neopixel import *
import sys
import os
import RPi.GPIO as GPIO
import config #Config.py holds user settings used by the various scripts

#****************************************************************************
#* User defined items to be set below - Make changes to config.py, not here *
#****************************************************************************
#Specific settings for on/off timer. Used to turn off LED's at night if desired.
#Verify Raspberry Pi is set to the correct time zone, otherwise the timer will be off.
usetimer = config.usetimer              #0 = No, 1 = Yes. Turn the timer on or off with this setting
offhour = config.offhour                #Use 24 hour time. Set hour to turn off display
offminutes = config.offminutes          #Set minutes to turn off display
onhour = config.onhour                  #Use 24 hour time. Set hour to turn on display
onminutes = config.onminutes            #Set minutes to on display

LED_COUNT = config.LED_COUNT      	#Number of LED pixels. Change this value to match the number of LED's being used on map

### If the next 8 variables are all set to 0, then the map will only show the basic Flight Category colors. i.e Version 1
### If the next 8 variables are all set to 1, then it will show lightning, high winds and reported weather. i.e. Version 3
hiwindblink = config.hiwindblink     	#1 = Yes, 0 = No. Blink the LED for high wind Airports.
lghtnflash = config.lghtnflash      	#1 = Yes, 0 = No. Flash the LED for an airport reporting severe weather like TSRA.
### if the next 6 variables are set to 0, but the previous 2 are set to 1, then map will display basic Flight Categories
### with blinking winds and lightning. i.e. Version 2
rainshow = config.rainshow	    	#1 = Yes, 0 = No. Change colors to denote rain reported.
frrainshow = config.frrainshow	    	#1 = Yes, 0 = No. Change colors to denote freezing rain reported.
snowshow = config.snowshow	    	#1 = Yes, 0 = No. Change colors to denote snow reported.
dustsandashshow = config.dustsandashshow 	#1 = Yes, 0 = No. Change colors to denote dust, sand, or ash reported.
fogshow = config.fogshow	    	#1 = Yes, 0 = No. Change colors to denote fog reported.
homeport = config.homeport		#1 = Yes, 0 = No. Turn on/off home airport feature. The home airport will use a marker color on every other pass
# if 'homeport = 1' be sure to set these variables appropriately
homeport_pin = config.homeport_pin	#Pin number of the home airport to display a marker color every other pass
homeport_display = config.homeport_display 	#2 = no color change, 1 = changing colors - user defined below by homeport_colors[], 0 = Solid color denoted by homeport_color below.
dim_value = config.dim_value		#Percentage of brightness to dim all other airports if homeport is being used. 0 = No dimming. 100 = completely off
# Misc settings
rgb_grb = config.rgb_grb		#1 = RGB color codes. 0 = GRB color codes. Populate color codes below with normal RGB codes and script will change if necessary
max_wind_speed = config.max_wind_speed  #In Knots. Any speed at or above will flash the LED for the appropriate airport if hiwindblink=1
update_interval = config.update_interval 	#Number of MINUTES between FAA updates - 15 minutes is a good compromise. A pushbutton switch can be used to force update.
dimmed_value = config.dimmed_value   	#Range is 0 - 255. This sets the value of LED brightness when light sensor detects low ambient light. Independent of homeport dimming.
bright_value = config.bright_value  	#Range is 0 - 255. This sets the value of LED brightness when light sensor detects high ambient light
iterations = config.iterations      	#Number of times that the rainbow animation will run before updating LED colors. 0 = disable rainbow display.
metar_age = config.metar_age        	#Metar Age in HOURS. This will pull the latest metar that has been published within the timeframe listed here.
		      			#If no Metar has been published within this timeframe, the LED will default to the color specified by color_nowx.
# Set Colors in RGB. Change numbers in paranthesis as desired. The order should be (Red,Green,Blue). This setup works for the WS2812 model of LED strips.
# WS2811 strips uses GRB colors, so change "rgb_grb = 0" above if necessary. Range is 0-255. (https://www.rapidtables.com/web/color/RGB_Color.html)
color_vfr = config.color_vfr              	#Full bright Green for VFR
color_mvfr = config.color_mvfr       		#Full bright Blue for MVFR
color_ifr = config.color_ifr              	#Full bright Red for IFR
color_lifr = config.color_lifr           	#Full bright Magenta for LIFR
color_nowx = config.color_nowx               	#No weather for NO WX
color_black = config.color_black              	#Black/Off
color_lghtn = config.color_lghtn          	#Full bright Yellow to represent lightning strikes
color_snow1 = config.color_snow1        	#White for Snow etc.
color_snow2 = config.color_snow2        	#Grey for Snow etc.
color_rain1 = config.color_rain1             	#Dark Blue for Rain etc.
color_rain2 = config.color_rain2            	#Blue for rain etc.
color_frrain1 = config.color_frrain1      	#Light Purple
color_frrain2 = config.color_frrain2       	#Purple for freezing rain etc.
color_dustsandash1 = config.color_dustsandash1 	#Tan/Brown for Dust and Sand
color_dustsandash2 = config.color_dustsandash2  #Dark Brown for Dust and Sand
color_fog1 = config.color_fog1            	#Silver for Fog
color_fog2 = config.color_fog2            	#Silver for Fog
color_homeport = config.color_homeport		#Color to denote home airport every other LED cycle. Used if 'homeport_display = 0'
homeport_colors = config.homeport_colors	#if 'homeport_display=1'. Change these colors as desired.
# Legend on/off. The setting 'legend' must be set to 1 for any legend LED's to be enabled. But you can disable the other
# legend types by defining it with a 0. No legend LED's, 5 basic legends LED's, 6, 7, 8, 9, 10, 11 or 12 total legend LED's can be used.
legend = config.legend          		#1 = Yes, 0 = No. Provides for basic vfr, mvfr, ifr, lifr, nowx legend. If 'legend=0' then no legends will be enabled.
legend_hiwinds = config.legend_hiwinds  	#1 = Yes, 0 = No. With this enabled high winds legend will be displayed.
legend_lghtn = config.legend_lghtn    		#1 = Yes, 0 = No. With this enabled Lightning/Thunderstorm legend will be displayed as well
legend_snow = config.legend_snow     		#1 = Yes, 0 = No. Snow legend
legend_rain = config.legend_rain     		#1 = Yes, 0 = No. Rain legend
legend_frrain = config.legend_frrain   		#1 = Yes, 0 = No. Freezing Rain legend
legend_dustsandash = config.legend_dustsandash 	#1 = Yes, 0 = No. Dust, Sand and/or Ash legend
legend_fog = config.legend_fog      		#1 = Yes, 0 = No. Fog legend
# Legend Pins assigned if used. Be sure that the 'airports' file has 'LGND' at these LED positions otherwise the legend will not display properly.
leg_pin_vfr = config.leg_pin_vfr  		#Set LED pin number for VFR Legend LED
leg_pin_mvfr = config.leg_pin_mvfr 		#Set LED pin number for MVFR Legend LED
leg_pin_ifr = config.leg_pin_ifr  		#Set LED pin number for IFR Legend LED
leg_pin_lifr = config.leg_pin_lifr 		#Set LED pin number for LIFR Legend LED
leg_pin_nowx = config.leg_pin_nowx 		#Set LED pin number for No Weather Legend LED
leg_pin_hiwinds = config.leg_pin_hiwinds 	#Set LED pin number for High Winds Legend LED
leg_pin_lghtn = config.leg_pin_lghtn   		#Set LED pin number for Thunderstorms Legend LED
leg_pin_snow = config.leg_pin_snow     		#Set LED pin number for Snow Legend LED
leg_pin_rain = config.leg_pin_rain     		#Set LED pin number for Rain Legend LED
leg_pin_frrain = config.leg_pin_frrain   	#Set LED pin number for Freezing Rain Legend LED
leg_pin_dustsandash = config.leg_pin_dustsandash #Set LED pin number for Dust/Sand/Ash Legend LED
leg_pin_fog = config.leg_pin_fog      		#Set LED pin number for Fog Legend LED

#************************************************************
#* End of User defined settings. Normally shouldn't change  *
#* any thing under here unless you are confident in change. *
#************************************************************

#LED Cycle times - Can change if necessary.
cycle0_wait = .9        #These cycle times all added together will equal the total amount of time the LED takes to finish displaying one cycle.
cycle1_wait = .9        #Each  cycle, depending on flight category, winds and weather reported will have various colors assigned.
cycle2_wait = .08       #For instance, VFR with 20 kts winds will have the first 3 cycles assigned Green and the last 3 Black for blink effect.
cycle3_wait = .1        #The cycle times then reflect how long each color cycle will stay on, producing blinking or flashing effects.
cycle4_wait = .08       #Lightning effect uses the short intervals at cycle 2 and cycle 4 to create the quick flash. So be careful if you change them.
cycle5_wait = .5

#List of METAR weather categories to designate weather in area. Many Metars will report multiple conditions, i.e. '-RA BR'.
#    The code pulls the first/main weather reported to compare against the lists below. In this example it uses the '-RA' and ignores the 'BR'.
#    See https://www.aviationweather.gov/metar/symbol for descriptions. Add or subtract codes as desired.
#    Thunderstorm and lightning
wx_lghtn_ck = ["TS", "TSRA", "TSGR", "+TSRA", "TSRG", "FC", "SQ", "VCTS", "VCTSRA", "VCTSDZ", "LTG"]
#    Snow in various forms
wx_snow_ck = ["BLSN", "DRSN", "-RASN", "RASN", "+RASN", "-SN", "SN", "+SN", "SG", "IC", "PE", "PL", "-SHRASN", "SHRASN", "+SHRASN", "-SHSN", "SHSN", "+SHSN"]
#    Rain in various forms
wx_rain_ck = ["-DZ", "DZ", "+DZ", "-DZRA", "DZRA", "-RA", "RA", "+RA", "-SHRA", "SHRA", "+SHRA", "VIRGA", "VCSH"]
#    Freezing Rain
wx_frrain_ck = ["-FZDZ", "FZDZ", "+FZDZ", "-FZRA", "FZRA", "+FZRA"]
#    Dust Sand and/or Ash
wx_dustsandash_ck = ["DU", "SA", "HZ", "FU", "VA", "BLDU", "BLSA", "PO", "VCSS", "SS", "+SS",]
#    Fog
wx_fog_ck = ["BR", "MIFG", "VCFG", "BCFG", "PRFG", "FG", "FZFG"]

#list definitions
cycle_wait = [cycle0_wait, cycle1_wait, cycle2_wait, cycle3_wait, cycle4_wait, cycle5_wait] #Used to create weather designation effects.
cycles = [0,1,2,3,4,5] #Used as a index for the cycle loop.
legend_pins = [leg_pin_vfr, leg_pin_mvfr, leg_pin_ifr, leg_pin_lifr, leg_pin_nowx, leg_pin_hiwinds, leg_pin_lghtn, leg_pin_snow, leg_pin_rain, leg_pin_frrain, leg_pin_dustsandash, leg_pin_fog] #Used to build legend display

#Setup for IC238 Light Sensor for LED Dimming, does not need to be commented out if sensor is not used, map will remain at full brightness.
#For more info on the sensor visit; http://www.uugear.com/portfolio/using-light-sensor-module-with-raspberry-pi/
GPIO.setmode(GPIO.BCM)  #set mode to BCM and use BCM pin numbering, rather than BOARD pin numbering.
GPIO.setup(4, GPIO.IN)  #set pin 4 as input for light sensor, if one is used. If no sensor used board remains at high brightness always.
GPIO.setup(22, GPIO.IN, pull_up_down=GPIO.PUD_UP) #set pin 22 to momentary push button to force FAA Weather Data update if button is used.

#LED strip configuration YOU MUST CHANGE LED_COUNT VALUE TO MATCH YOUR SETUP:
#LED_COUNT      = 60      #Number of LED pixels. Change this value to match the number of LED's being used on map
LED_PIN        = 18      #GPIO pin connected to the pixels (18 uses PWM!).
#LED_PIN        = 10     #GPIO pin connected to the pixels (10 uses SPI /dev/spidev0.0).
LED_FREQ_HZ    = 800000  #LED signal frequency in hertz (usually 800khz)
LED_DMA        = 5       #DMA channel to use for generating signal (try 5)
LED_INVERT     = False   #True to invert the signal (when using NPN transistor level shift)
LED_CHANNEL    = 0       #set to '1' for GPIOs 13, 19, 41, 45 or 53
LED_STRIP      = ws.WS2811_STRIP_GRB #Strip type and color ordering
LED_BRIGHTNESS = 255     #starting brightness. It will be changed below.

#Timer calculations
now = datetime.now() #Get current time
timeoff = now.replace(hour=offhour, minute=offminutes, second=0, microsecond=0) #When to turn map off
timeon = now.replace(hour=onhour, minute=onminutes, second=0, microsecond=0)	#When to turn map back on
diff = timeon - timeoff
time_delay = 10 #number of seconds for delay in internet check routine

#Create an instance of NeoPixel
strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, LED_FREQ_HZ, LED_DMA, LED_INVERT, LED_BRIGHTNESS, LED_CHANNEL, LED_STRIP)
strip.begin()

#Functions
#Rainbow Animation functions - taken from https://github.com/JJSilva/NeoSectional/blob/master/metar.py
def wheel(pos):
        """Generate rainbow colors across 0-255 positions."""
        if pos < 85:
                return Color(pos * 3, 255 - pos * 3, 0)
        elif pos < 170:
                pos -= 85
                return Color(255 - pos * 3, 0, pos * 3)
        else:
                pos -= 170
                return Color(0, pos * 3, 255 - pos * 3)

def rainbowCycle(strip, iterations, wait_ms=2):
        """Draw rainbow that uniformly distributes itself across all pixels."""
        for j in range(256*iterations):
                for i in range(strip.numPixels()):
                        strip.setPixelColor(i, wheel((int(i * 256 / strip.numPixels()) + j) & 255))
                strip.show()
                #time.sleep(wait_ms/1000.0)

def turnoff(strip): #used for the timer routine to turn off th LED's at night.
        for i in range(strip.numPixels()):
                strip.setPixelColor(i, Color(0,0,0))
        strip.show()


#Reduces the brightness of the colors for every airport except for the "homeport_pin" designated airport, which remains at the brightness set by
#"bright_value" above in user setting. "data" is the airport color to display and "value" is the percentage of the brightness to be dimmed.
#For instance if full bright white (255,255,255) is provided and the desired dimming is 50%, then the color returned will be (128,128,128),
#or half as bright. The dim_value is set in the user defined area.
def dim(data,value):

	red = data[0] - ((value * data[0])/100)
	if red < 0:
		red = 0

	grn = data[1] - ((value * data[1])/100)
	if grn < 0:
		grn = 0

	blu = data[2] - ((value * data[0])/100)
	if blu < 0:
		blu = 0

	data =[red,grn,blu]
	return data

#Change color code to work with various led strips. For instance, WS2812 model strip uses RGB where WS2811 model uses GRB
#Set the "rgb_grb" user setting above. 1 for RGB LED strip, and 0 for GRB strip.
def rgbtogrb(data,order=0):
        red = data[0]
        grn = data[1]
        blu = data[2]

	if order:
        	data = [red,grn,blu]
        else:
                data =[grn,red,blu]
	return data

#Start of executed code
toggle = 0 #used for homeport display
outerloop = 1 #Set to TRUE for infinite outerloop
while (outerloop):

	#Dictionary definitions. Need to reset whenever new weather is received
	stationiddict = {"":""}
	windsdict = {"":""}
	wxstringdict = {"":""}

	rainbowCycle(strip, iterations) #Display Rainbow Pattern to signify updating. Set iterations to 0 to disable.

	#read airports file - read each time weather is updated in case a change to "airports" file was made while script was running.
	with open("/NeoSectional/airports") as f:
        	airports = f.readlines()
	airports = [x.strip() for x in airports]

	#Define URL to get weather METARS. This will pull only the latest METAR from the last 2.5 hours. If no METAR reported withing the last 2.5 hours, Airport LED will be white.
	url = "https://www.aviationweather.gov/adds/dataserver_current/httpparam?dataSource=metars&requestType=retrieve&format=xml&mostRecentForEachStation=constraint&hoursBeforeNow="+str(metar_age)+"&stationString="

	#Build URL to submit to FAA with the proper airports from the airports file
	for airportcode in airports:
        	if airportcode == "NULL" or airportcode == "LGND": #3.01 bug fixed by adding test for "LGND"
                	continue
        	url = url + airportcode + ","
     	url = url[:-1] #strip trailing comma from string

#	url = "http://192.168.86.25:8887/airports.xml" #Dummy xml file to test different weather conditions for debugging.

#	try: #Simple Error trap, in case FAA web site is not responding
#        	content = urllib2.urlopen(url).read()
#	except: #End of Error trap
#        	pass

	while True: #Check internet availability and retry if necessary
    		try:
			content = urllib2.urlopen(url).read()
        		print ('Internet Available')
        		break
    		except:
        		print ('internet not available')
        		time.sleep(10)
        		pass

	root = ET.fromstring(content) #Process XML data returned from FAA

	#grab the airport category, wind speed and various weather from the results given from FAA.
	for metar in root.iter('METAR'):
        	stationId = metar.find('station_id').text

		# Routine to create flight category via cloud cover and/or visability when flight category is not reported.
		# Routine written and contributed to project by Nick Cirincione. Thank you for your contribution.
		if metar.find('flight_category') is None: #if category is blank, then see if there's a sky condition or vis that would dictate flight category
			flightcategory = "VFR" #intialize flight category
			print (stationId + " Not Reporting Flight Category.")

	                #There can be multiple layers of clouds in each METAR, but they are always listed lowest AGL first.
        	        #Check the lowest (first) layer and see if it's overcast, broken, or obscured. If it is, then compare to cloud base height to set flight category.
                	#This algorithm basically sets the flight category based on the lowest OVC, BKN or OVX layer.
	                for sky_condition in metar.iter('sky_condition'):   #for each sky_condition from the XML
				sky_cvr = sky_condition.attrib['sky_cover']     #get the sky cover (BKN, OVC, SCT, etc)
                	        if sky_cvr in ("OVC","BKN","OVX"): #If the layer is OVC, BKN or OVX, set Flight category based on height AGL
					cld_base_ft_agl = sky_condition.attrib['cloud_base_ft_agl'] #get cloud base AGL from XML
                                	cld_base_ft_agl = int(cld_base_ft_agl)                      #convert string to integer
	                                if cld_base_ft_agl < 500:
						flightcategory = "LIFR"
                	                        break

                        	        elif 500 <= cld_base_ft_agl < 1000:
                                	        flightcategory = "IFR"
                                        	break

	                                elif 1000 <= cld_base_ft_agl <= 3000:
        	                                flightcategory = "MVFR"
                	                        break

                        	        elif cld_base_ft_agl > 3000:
                                	        flightcategory = "VFR"
                                        	break

	                #visibilty can also set flight category. If the clouds haven't set the fltcat to LIFR. See if visibility will
			if flightcategory != "LIFR": #if it's LIFR due to cloud layer, no reason to check any other things that can set flight category.
				if metar.find('visibility_statute_mi') is not None: #check XML if visibility value exists
					visibility_statute_mi = metar.find('visibility_statute_mi').text   #get visibility number
                              		visibility_statute_mi = float(visibility_statute_mi)               #convert string to float

	                              	if visibility_statute_mi < 1.0:
        	                              	flightcategory = "LIFR"

                	              	elif 1.0 <= visibility_statute_mi < 3.0:
                        	              	flightcategory = "IFR"

                              		elif 3.0 <= visibility_statute_mi <= 5.0 and flightcategory != "IFR":  #if Flight Category was already set to IFR by clouds, it can't be reduced to MVFR
                                      		flightcategory = "MVFR"

			print (stationId + " flight category is " + flightcategory)
		else:
			flightcategory = metar.find('flight_category').text  #pull flight category if it exists and save all the algoritm above
		#End of added routine to create flight category via cloud cover and/or visability when flight category is not reported.

		#grab wind speeds from returned FAA data
		if metar.find('wind_speed_kt') is None: #if wind speed is blank, then bypass
			windspeedkt = 0
		else:
			windspeedkt = metar.find('wind_speed_kt').text

		#grab Weather info from returned FAA data
        	if metar.find('wx_string') is None: #if weather string is blank, then bypass
                	wxstring = "NONE"
		else:
	        	wxstring = metar.find('wx_string').text

		#Check for duplicate airport identifier and skip if found, otherwise store in dictionary. covers for dups in "airports" file
        	if stationId in stationiddict:
                	print ("Duplicate, only saved first metar category")
        	else:
                	stationiddict[stationId] = flightcategory #build category dictionary

		if stationId in windsdict:
			print ("Duplicate, only saved the first winds")
		else:
			windsdict[stationId] = windspeedkt #build windspeed dictionary

        	if stationId in wxstringdict:
                	print ("Duplicate, only saved the first weather")
        	else:
                	wxstringdict[stationId] = wxstring #build weather dictionary

	#Setup timed loop for updating FAA Weather that will run based on the value of update_interval which is a user setting
	timeout_start = time.time() #Start the timer. When timer hits user-defined value, go back to outer loop to update FAA Weather.
	while time.time() < timeout_start + (update_interval * 60): #take "update_interval" which is in minutes and turn into seconds

                #Timer routine, used to turn off LED's at night if desired. Use 24 hour time in settings.
                if usetimer: #check to see if the user wants to use a timer.
			if diff.days < 0: #this checks to see if the timeoff is before midnight and timeon is after (typical) and adjust.
        			diff = timedelta(days=0, seconds=diff.seconds, microseconds=diff.microseconds)

			end_time = timeoff + diff
			if timeoff <= datetime.now() <= end_time:
        	                sys.stdout.write ("\n\033[1;34;40m LEDs Sleeping-  ") #Escape codes to render Blue text on screen
                	        sys.stdout.flush ()
                        	turnoff(strip)

				while timeoff <= datetime.now() <= end_time:
                                	sys.stdout.write ("z")
                                	sys.stdout.flush ()
                                	time.sleep(1)
                        	print ("\033[0;0m\n") #Turn off Blue text.

                #Check to see if pushbutton is pressed to force an update of FAA Weather
                #If no button is connected, then this is bypassed and will only update when 'update_interval' is met
                if GPIO.input(22) == False:
                        print ('Breaking out of loop to refresh FAA Data')
                        break

		#Bright light will provide a low state (0) on GPIO. Dark light will provide a high state (1).
		#Full brightness will be used if no light sensor is installed.
		if GPIO.input(4) == 1:
    			LED_BRIGHTNESS = dimmed_value
		else:
    			LED_BRIGHTNESS = bright_value
		strip.setBrightness(LED_BRIGHTNESS)

		toggle = not(toggle) #Used to determine if the homeport color should be displayed if "homeport = 1"

		#Start main loop. This loop will create all the necessary colors to display the weather one time.
		for cycle_num in cycles: #cycle through the strip 6 times, setting the color then displaying to create various effects.
			print ("\n Cycle Num = " + str(cycle_num)) #debug

			i = 0 #Inner Loop. Increments through each LED in the strip setting the appropriate color to each individual LED.
			for airportcode in airports:
                		flightcategory = stationiddict.get(airportcode,"NONE") #Pull the next flight category from dictionary.
                		airportwinds = windsdict.get(airportcode,0) #Pull the winds from the dictionary.
                		airportwx_long = wxstringdict.get(airportcode,"NONE") #Pull the weather reported for the airport from dictionary.
				airportwx = airportwx_long.split(" ",1)[0] #Grab only the first parameter of the weather reported.

				#Check to see if airport code is a NULL and set to black.
				if airportcode == "NULL":
					color = color_black

				#Build and display Legend. "legend" must be set to 1 in the user defined section and "LGND" set in airports file.
				if legend and airportcode == "LGND" and (i in legend_pins):
					if i == leg_pin_vfr:
						color = color_vfr

                                        if i == leg_pin_mvfr:
						color = color_mvfr

                                        if i == leg_pin_ifr:
						color = color_ifr

                                        if i == leg_pin_lifr:
						color = color_lifr

                                        if i == leg_pin_nowx:
                                                color = color_nowx

                                        if i == leg_pin_hiwinds and legend_hiwinds:
                                        	if (cycle_num == 3 or cycle_num == 4 or cycle_num == 5):
                                                	color = color_black
						else:
							color=color_ifr

                                        if i == leg_pin_lghtn and legend_lghtn:
                                                if (cycle_num == 2 or cycle_num == 4): #Check for Thunderstorms
                                                        color = color_lghtn
						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 3 or cycle_num == 5):
							color=color_mvfr

                                        if i == leg_pin_snow and legend_snow:
                                                if (cycle_num == 3 or cycle_num == 5): #Check for Snow
                                                        color = color_snow1

                                                if (cycle_num == 4):
                                                        color = color_snow2

						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 2):
							color=color_lifr

                                        if i == leg_pin_rain and legend_rain:
                                                if (cycle_num == 3 or cycle_num == 5): #Check for Rain
                                                        color = color_rain1

                                                if (cycle_num == 4):
                                                        color = color_rain2

						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 2):
							color=color_vfr

                                        if i == leg_pin_frrain and legend_frrain:
                                                if (cycle_num == 3 or cycle_num == 5): #Check for Freezing Rain
                                                        color = color_frrain1

                                                if (cycle_num == 4):
                                                        color = color_frrain2

						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 2):
							color = color_mvfr

                                        if i == leg_pin_dustsandash and legend_dustsandash:
                                                if (cycle_num == 3 or cycle_num == 5): #Check for Dust, Sand or Ash
                                                        color = color_dustsandash1

                                                if (cycle_num == 4):
                                                        color = color_dustsandash2

						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 2):
							color=color_vfr

                                        if i == leg_pin_fog and legend_fog:
                                                if (cycle_num == 3 or cycle_num == 5): #Check for Fog
                                                        color = color_fog1

                                                if (cycle_num == 4):
                                                        color = color_fog2

						elif (cycle_num == 0 or cycle_num == 1 or cycle_num == 2):
							color=color_ifr

				#Start of weather display code for each airport in the "airports" file
				#Check flight category and set the appropriate color to display
				if  flightcategory != "NONE":
					if flightcategory == "VFR": 	#Visual Flight Rules
						color = color_vfr
					elif flightcategory == "MVFR": 	#Marginal Visual Flight Rules
						color = color_mvfr
					elif flightcategory == "IFR": 	#Instrument Flight Rules
						color = color_ifr
					elif flightcategory == "LIFR": 	#Low Instrument Flight Rules
						color = color_lifr

				elif flightcategory == "NONE" and airportcode != "LGND" and airportcode != "NULL": #3.01 bug fix by adding "LGND" test
					color = color_nowx #No Weather reported.

				#Check winds and set the 2nd half of cycles to black to create blink effect
				if hiwindblink: #bypass if "hiwindblink" is set to 0
					if (int(airportwinds) >= max_wind_speed and (cycle_num == 3 or cycle_num == 4 or cycle_num == 5)):
						color = color_black
						print (airportcode + " Winds = " + airportwinds + " ") #debug

				#Check the wxstring from FAA for reported weather and create color changes in LED for weather effect.
				if airportwx != "NONE":
					if  lghtnflash:
						if (airportwx in wx_lghtn_ck and (cycle_num == 2 or cycle_num == 4)): #Check for Thunderstorms
							color = color_lghtn

					if snowshow:
						if (airportwx in wx_snow_ck and (cycle_num == 3 or cycle_num == 5)): #Check for Snow
                                			color = color_snow1

                                		if (airportwx in wx_snow_ck and cycle_num == 4):
                                        		color = color_snow2

					if rainshow:
						if (airportwx in wx_rain_ck and (cycle_num == 3 or cycle_num == 4)): #Check for Rain
                                			color = color_rain1

                                		if (airportwx in wx_rain_ck and cycle_num == 5):
                                        		color = color_rain2

					if frrainshow:
						if (airportwx in wx_frrain_ck and (cycle_num == 3 or cycle_num == 5)): #Check for Freezing Rain
                                			color = color_frrain1

                                		if (airportwx in wx_frrain_ck and cycle_num == 4):
                                        		color = color_frrain2

					if dustsandashshow:
						if (airportwx in wx_dustsandash_ck and (cycle_num == 3 or cycle_num == 5)): #Check for Dust, Sand or Ash
                                			color = color_dustsandash1

                                		if (airportwx in wx_dustsandash_ck and cycle_num == 4):
                                        		color = color_dustsandash2

					if fogshow:
						if (airportwx in wx_fog_ck and (cycle_num == 3 or cycle_num == 5)): #Check for Fog
                                			color = color_fog1

                                                if (airportwx in wx_fog_ck and cycle_num == 4):
                                                        color = color_fog2

				#If homeport is set to 1 then turn on the appropriate LED using a specific color, This will toggle
				#so that every other time through, the color will display the proper weather, then homeport color(s).
				if i == homeport_pin and homeport and toggle:
					if homeport_display == 1:
						color = homeport_colors[cycle_num]
					elif homeport_display == 2:
						pass
					else:
						color = color_homeport

				xcolor = rgbtogrb(color,rgb_grb) #Check and change color code for RGB or GRB format

				if i == homeport_pin and homeport: #if this is the home airport, don't dim out the brightness
                                        norm_color = xcolor
                                        xcolor = Color(norm_color[0], norm_color[1], norm_color[2])
				elif homeport: #if this is not the home airport, dim out the brightness
                                        dim_color = dim(xcolor,dim_value)
                                        xcolor = Color(dim_color[0], dim_color[1], dim_color[2])
				else: #if home airport feature is disabled, then don't dim out any airports brightness
                                        norm_color = xcolor
                                        xcolor = Color(norm_color[0], norm_color[1], norm_color[2])

				strip.setPixelColor(i, xcolor) #set color to display on a specific LED for the current cycle_num cycle.
				i = i + 1 #set next LED in strip

                	strip.show() #Display strip with newly assigned colors for the current cycle_num cycle.
			wait_time = cycle_wait[cycle_num] #cycle_wait time is a user defined value
			time.sleep(wait_time) #pause between cycles. pauses are setup in user definitions.
